/* FILE: avftovio.cc                 -*-Mode: c++-*-
 *
 * Program to try to convert from Any (known) Vector Field file
 * to a vio vector field file format.
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "oc.h"
#include "nb.h"
#include "vf.h"

/* End includes */

static char *inname;
static Vf_Mesh *mesh;

void Usage(void)
{
  fprintf(stderr,"Usage: avftovio infile >outfile\n");
  exit(1);
}

extern "C" {
static int
MyTcl_AppInit(Tcl_Interp *interp)
{
    Oc_AutoBuf ab;
    if (Oc_Init(interp) == TCL_ERROR) {
        return TCL_ERROR;
    }
    if (Vf_Init(interp) == TCL_ERROR) {
        return TCL_ERROR;
    }

    // Fill mesh from input file
    Vf_FileInput *vffreader = Vf_FileInput::NewReader(inname);
    if(vffreader==NULL) { // Unknown file type or other error
        fprintf(stderr,"ERROR detected processing file %s\n",inname);
        return Tcl_GlobalEval(interp, ab("exit 1"));
    }
    else { // Reader found for inname
        mesh=vffreader->NewMesh();
        delete vffreader;
        if(mesh==NULL) {
            fprintf(stderr,"ERROR detected while reading file %s\n",inname);
            return Tcl_GlobalEval(interp, ab("exit 2"));
        }
    }

    // Write vio file to stdout
    Vf_VioFileOutput vfo;
    vfo.WriteMesh(mesh,stdout,NULL);

    // That's all folks!
    fflush(stdout);
    return Tcl_GlobalEval(interp, ab("exit 0"));
}
}

int Oc_Main(int argc,char **argv)
{
    if(argc!=2) Usage();
    inname=argv[1];

    Tcl_Main(argc, argv, MyTcl_AppInit);
    return 0;
}
