# FILE: makerules.tcl
#
# This file controls the application 'pimake' by defining rules which
# describe how to build the applications and/or extensions produced by
# the source code in this directory.
#
# Verify that this script is being sourced by pimake
if {[llength [info commands MakeRule]] == 0} {
    error "'[info script]' must be evaluated by pimake"
}
#
########################################################################

set ocObjects {
	autobuf
	messages
	nullchannel
	oc
}

MakeRule Define {
    -targets            all
    -dependencies       [concat configure [Platform StaticLibrary oc]]
}

MakeRule Define {
    -targets            configure
    -dependencies       [Platform Specific port.h]
}

MakeRule Define {
    -targets		[Platform Specific port.h]
    -dependencies	[concat procs.tcl [Platform Executables varinfo]]
    -script		{Oc_MakePortHeader [Platform Executables varinfo] \
			        [Platform Specific port.h]
			}
}

set acceptslm 0
if {![catch {[Oc_Config RunPlatform] GetValue TCL_LIBS} _]} {
    foreach _ [split $_] {
	if {[string match -lm $_]} {
	    set acceptslm 1
	    break
	}
    }
}

if {$acceptslm} {
MakeRule Define {
    -targets		[Platform Executables varinfo]
    -dependencies	[Platform Objects varinfo]
    -script		{Platform Link -obj varinfo -sub CONSOLE \
			-[[Oc_Config RunPlatform] GetValue platform_name] \
			{-lm} -out varinfo
			} 
}
} else {
MakeRule Define {
    -targets		[Platform Executables varinfo]
    -dependencies	[Platform Objects varinfo]
    -script		{Platform Link -obj varinfo -sub CONSOLE -out varinfo} 
}
}
unset acceptslm

MakeRule Define {
    -targets		[Platform Objects varinfo]
    -dependencies	[concat [list [Platform Name]] \
			        [[CSourceFile New _ varinfo.cc] Dependencies]]
    -script		{Platform Compile C++ -opt 1 \
			        -inc [[CSourceFile New _ varinfo.cc] DepPath] \
			        -src varinfo.cc -out varinfo
			}
}

MakeRule Define {
    -targets            [Platform Name]
    -dependencies       {}
    -script             {MakeDirectory [Platform Name]}
}

MakeRule Define {
    -targets            tclIndex
    -dependencies       [glob -nocomplain *.tcl]
    -script             {
                         MakeTclIndex .
                        }
}

MakeRule Define {
    -targets            [Platform StaticLibrary oc]
    -dependencies	[concat tclIndex [Platform Objects $ocObjects]]
    -script		[format {
			    eval file delete [Platform StaticLibrary oc]
			    Platform MakeLibrary -out oc -wintel user32.lib \
			            -winalp user32.lib -obj {%s}
			    Platform IndexLibrary oc 
			} $ocObjects]
}

MakeRule Define {
    -targets		[Platform Objects autobuf]
    -dependencies	[concat [list [Platform Name]] \
			        [[CSourceFile New _ autobuf.cc] Dependencies]]
    -script		{Platform Compile C++ -opt 1 \
			        -inc [[CSourceFile New _ autobuf.cc] DepPath] \
			        -src autobuf.cc -out autobuf
			}
}

MakeRule Define {
    -targets		[Platform Objects nullchannel]
    -dependencies	[concat [list [Platform Name]] \
			        [[CSourceFile New _ nullchannel.cc] Dependencies]]
    -script		{Platform Compile C++ -opt 1 \
			        -inc [[CSourceFile New _ nullchannel.cc] DepPath] \
			        -src nullchannel.cc -out nullchannel
			}
}

# The next rule is uglier than I would like because of a "chicken
# and egg" problem with port.h .
MakeRule Define {
    -targets		[Platform Objects oc]
    -dependencies	[concat [list [Platform Name]] oc.cc \
			        oc.h [Platform Specific port.h] \
				messages.h functions.h \
			        [[CSourceFile New _ imports.h] Dependencies]]
    -script		{Platform Compile C++ -opt 1 \
			        -inc [[CSourceFile New _ oc.cc] DepPath] \
			        -src oc.cc -out oc 
			}
}

# Doubly ugly.  port.h problem plus some compilers have broken
# optimizations we have to work around
if {[catch {
    [Oc_Config RunPlatform] GetValue \
            program_compiler_c++_property_optimization_breaks_varargs
} _] || !$_} {
MakeRule Define {
    -targets		[Platform Objects messages]
    -dependencies	[concat [list [Platform Name]] messages.cc \
			        oc.h [Platform Specific port.h] \
				messages.h functions.h \
			        [[CSourceFile New _ imports.h] Dependencies]]
    -script		{Platform Compile C++ -opt 1 \
			        -inc [[CSourceFile New _ messages.cc] DepPath] \
			        -src messages.cc -out messages
			}
}
} else {
MakeRule Define {
    -targets		[Platform Objects messages]
    -dependencies	[concat [list [Platform Name]] messages.cc \
			        oc.h [Platform Specific port.h] \
				messages.h functions.h \
			        [[CSourceFile New _ imports.h] Dependencies]]
    -script		{Platform Compile C++ -opt 0 \
			        -inc [[CSourceFile New _ messages.cc] DepPath] \
			        -src messages.cc -out messages
			}
}
}

MakeRule Define {
    -targets            distclean
    -dependencies       clean
    -script             {DeleteFiles [Platform Specific port.h]
			 DeleteFiles [Platform Executables varinfo]
			 DeleteFiles [Platform Name]}
}
 
MakeRule Define {
    -targets            clean
    -dependencies       mostlyclean
}
 
MakeRule Define {
    -targets            mostlyclean
    -dependencies       objclean
    -script             {eval DeleteFiles [Platform StaticLibrary oc]}
}
 
MakeRule Define {
    -targets            objclean
    -dependencies       {}
    -script             [format {
			    eval DeleteFiles [Platform Objects {varinfo %s}]
			} $ocObjects]
}

unset ocObjects
